use std::sync::Arc;
use tokio::sync::Mutex;

use super::led_color::LedColor;

#[derive(Clone)]
pub struct Screenshot {
    bitmap: Arc<Mutex<Option<Vec<u8>>>>,
    width: usize,
    height: usize,
    led_number_of_x: usize,
    led_number_of_y: usize,
}

impl Screenshot {
    pub fn new(width: usize, height: usize) -> Self {
        Self {
            bitmap: Arc::new(Mutex::new(None)),
            led_number_of_x: 0,
            led_number_of_y: 0,
            width,
            height,
        }
    }

    pub fn get_size(&self) -> (usize, usize) {
        (self.width, self.height)
    }

    pub fn get_number_of_leds(&self) -> (usize, usize) {
        (self.led_number_of_x, self.led_number_of_y)
    }

    pub fn set_number_of_leds(&mut self, led_number_of_x: usize, led_number_of_y: usize) {
        self.led_number_of_x = led_number_of_x;
        self.led_number_of_y = led_number_of_y;
    }
    pub async fn get_top_colors(&self) -> anyhow::Result<Vec<LedColor>> {
        self.get_x_colors(XPosition::Top).await
    }
    pub async fn get_bottom_colors(&self) -> anyhow::Result<Vec<LedColor>> {
        self.get_x_colors(XPosition::Bottom).await
    }

    async fn get_x_colors(&self, position: XPosition) -> anyhow::Result<Vec<LedColor>> {
        if self.led_number_of_x == 0 {
            return Ok(vec![]);
        }

        let bitmap = self.bitmap.lock().await;
        match bitmap.as_ref() {
            Some(bitmap) => {
                let cell_size_x = self.width / self.led_number_of_x;
                let cell_size_y = self.height / 5;
                let cell_size = cell_size_x * cell_size_y;
                let y_range = match position {
                    XPosition::Top => 0..cell_size_y,
                    XPosition::Bottom => self.height - cell_size_y..self.height,
                };

                let mut colors = vec![LedColor::default(); self.led_number_of_x];
                let stride = bitmap.len() / self.height;

                for pos in 0..self.led_number_of_x {
                    let mut r = 0u32;
                    let mut g = 0u32;
                    let mut b = 0u32;
                    for x in pos * cell_size_x..(pos + 1) * cell_size_x {
                        for y in y_range.to_owned() {
                            let i = stride * y + 4 * x;
                            r += bitmap[i + 2] as u32;
                            g += bitmap[i + 1] as u32;
                            b += bitmap[i] as u32;
                        }
                    }
                    colors[pos] = LedColor::new(
                        (r / cell_size as u32) as u8,
                        (g / cell_size as u32) as u8,
                        (b / cell_size as u32) as u8,
                    );
                }
                return Ok(colors);
            }
            None => Ok(vec![]),
        }
    }

    pub async fn set_bitmap(&mut self, bitmap: Vec<u8>) {
        let mut self_bitmap = self.bitmap.lock().await;
        *self_bitmap = Some(bitmap);
    }

    pub async fn to_webp_base64(&self) -> String {
        let bitmap = self.bitmap.lock().await;
        match bitmap.to_owned() {
            Some(bitmap) => {
                let mut bitflipped = Vec::with_capacity(self.width * self.height * 3);
                let stride = bitmap.len() / self.height;

                for y in 0..self.height {
                    for x in 0..self.width {
                        let i = stride * y + 4 * x;
                        bitflipped.extend_from_slice(&[bitmap[i + 2], bitmap[i + 1], bitmap[i]]);
                    }
                }

                let webp_memory = webp::Encoder::from_rgb(
                    bitflipped.as_slice(),
                    self.width as u32,
                    self.height as u32,
                )
                .encode(100.0);
                return base64::encode(&*webp_memory);
            },
            None => {
                "".to_owned()
            }
        }
    }
}

enum XPosition {
    Top,
    Bottom,
}
